// ================================================
// WidgetPreference
// ================================================
function WidgetPreference() {
	LOG__("Object: WidgetPreference");
	if (!window.widget)
		LOG__("Not found window.widget");

	this.jacketsNum = 1;
	this.defaultPrefs = {
		clock:  15*60*60*1000,  // 15:00
		clockA: 10*60*60*1000,  // 10:00
		clockB: 15*60*60*1000,  // 15:00
		alarmMode: false,
		jacket: 0
	}
	this.prefs = {};
	for (key in this.defaultPrefs)
		this.prefs[key] = this.defaultPrefs[key];

	if (window.widget)
		this.keyPrefix = "widgetID-" + widget.identifier;
}

// Load preferences
WidgetPreference.prototype.load = function() {
	LOG2__("WidgetPreference.load");
	if (!window.widget) return null;

	var key, newValue;
	for (key in this.prefs) {
		newValue = widget.preferenceForKey(this.keyPrefix + "-" + key);
		if (newValue == null)
			newValue = widget.preferenceForKey(key);
		this.prefs[key] = this.correctValue(newValue, key);
	}
}

// Get the preference for key
WidgetPreference.prototype.value = function(key) {
//	LOG2__("WidgetPreference.value:" + key);
	return this.prefs[key];
}

// Set the preference for key
WidgetPreference.prototype.setValue = function(value, key) {
	LOG2__("WidgetPreference.setValue:" + value + " forKey:" + key);
	var newValue = this.correctValue(value, key);

	if (newValue != null) {
		if (this.prefs[key] != newValue) {
			this.prefs[key] = newValue;
			if (window.widget) {
				widget.setPreferenceForKey(newValue, this.keyPrefix + "-" + key);
				widget.setPreferenceForKey(newValue, key);
			}
		}
		return true;
	} else {
		return false;
	}
}

// Remove preferences
WidgetPreference.prototype.removeMyPref = function() {
	LOG2__("WidgetPreference.removeMyPref:" + this.keyPrefix);
	if (!window.widget) return null;
	var key;

	for (key in this.prefs)
		widget.setPreferenceForKey(null, this.keyPrefix + "-" + key);
}

// Verify and correct the value
WidgetPreference.prototype.correctValue = function(value, key) {
//	LOG2__("WidgetPreference.correctValue:" + value + " forKey:" + key);
	var correct = true;
	var correctedValue = value;

	switch(key) {
		case "clock":
		case "clockA":
		case "clockB":
			if ((value != Math.floor(value)) || (value < 0) || (24*60*60*1000 < value))
				correct = false;
			break;
		case "alarmMode":
			if (!((value == true) || (value == false)))
				correct = false;
			break;
		case "jacket":
			if ((value != Math.floor(value)) || (value < 0) || (this.jacketsNum <= value))
				correct = false;
			break;
		default:
			return null;
	}

	if (!correct) {
		correctedValue = this.defaultPrefs[key];
		if (window.widget) {
			widget.setPreferenceForKey(correctedValue, this.keyPrefix + "-" + key);
			widget.setPreferenceForKey(correctedValue, key);
		}
	}
	return correctedValue;
}


// ================================================
// PreferencePane
// ================================================
function PreferencePane(widgetPreference) {
	LOG__("Object: PreferencePane");
	this.widgetPref = widgetPreference;
	this.prefs = {};
}

// Get the preference for key
PreferencePane.prototype.value = function(key) {
//	LOG2__("PreferencePane.value:" + key);
	return this.prefs[key];
}

// Set the preference for key
PreferencePane.prototype.setValue = function(value, key) {
	var newValue = this.widgetPref.correctValue(value, key);
	this.prefs[key] = newValue;
	switch(key) {
		case "jacket":
			this.setJacket(newValue);
			break;
	}
}

// Apply preferences
PreferencePane.prototype.apply = function() {
	for (key in this.prefs) {
		if (this.prefs[key] != this.widgetPref.prefs[key]) {
			switch(key) {
				case "jacket":
					setJacket(this.prefs[key]);
					break;
			}
		}
	}
	this.prefs = {};
}

PreferencePane.prototype.changeJacket = function(rev) {
	var newJacket;
	if (!rev) {
		newJacket = this.value("jacket") + 1;
		if (this.widgetPref.jacketsNum <= newJacket )
			newJacket = 0;
	} else {
		newJacket = this.value("jacket") - 1;
		if (newJacket < 0)
			newJacket = this.widgetPref.jacketsNum - 1;
	}
	this.setValue(newJacket, "jacket");
}

PreferencePane.prototype.setJacket = function(newJacket) {
    var colorCode = jacketFiles[newJacket];
	document.getElementById("jacketSampleImage").src = "Images/jacket" + colorCode + ".png";
}
